from nekbot.protocols.base.group_chat import GroupChats

__author__ = 'nekmo'

from nekbot.protocols import GroupChat

class GroupChatTelegram(GroupChat):
    def __init__(self, protocol, name, id):
        GroupChat.__init__(self, protocol, name)
        self.id = id

    def send_message(self, body):
        self.protocol.tg.msg('chat#%i' % self.id, self.protocol.prepare_message(body))

class GroupChatsTelegram(GroupChats):
    pass


from logging import getLogger
import os
import pytg
from pytg.tg import message as tg_message
from pytg.utils import coroutine, broadcast
from nekbot.protocols import Protocol
from nekbot.protocols.telegram.group_chat import GroupChatsTelegram
from nekbot.protocols.telegram.message import MessageTelegram

__author__ = 'nekmo'
__dir__ = os.path.dirname(__file__)

TELEGRAM_BIN = os.path.abspath(os.path.join(__dir__, 'bin/telegram-cli'))
TELEGRAM_PUB = os.path.abspath(os.path.join(__dir__, 'bin/server.pub'))


logger = getLogger('nekbot.protocols.telegram')


class Telegram(Protocol):
    def init(self):
        self.groupchats = GroupChatsTelegram(self)
        self.bot = None # User Bot
        self.tg = pytg.Telegram(TELEGRAM_BIN, TELEGRAM_PUB)
        # Create processing pipeline
        pipeline = broadcast([
            tg_message(self.input_message(self.tg))
        ])
        self.tg.register_pipeline(pipeline)
        # Start telegram cli
        self.tg.start()

    def prepare_message(self, body):
        if not isinstance(body, (str, unicode)):
            body = str(body)
        try:
            body = body.decode('utf-8')
        except:
            pass
        return body

    @coroutine
    def input_message(self, tg):
        # To avoid ping flood attack, we'll respond to ping once every 10 sec
        try:
            while True:
                msg = (yield)
                self.propagate('message', MessageTelegram(self, msg))
        except GeneratorExit:
            pass

    def run(self):
        while True:
            # Keep on polling so that messages will pass through our pipeline
            self.tg.poll()

    def close(self):
        logger.debug('Closing Telegram...')
        self.tg.safe_quit()


from logging import getLogger
from datetime import datetime, timedelta
from nekbot.protocols.telegram.group_chat import GroupChatTelegram
from nekbot.protocols.telegram.user import UserTelegram
from nekbot.protocols import Message

__author__ = 'nekmo'

logger = getLogger('nekbot.protocols.telegram.message')


class MessageTelegram(Message):
    def __init__(self, protocol, msg):
        logger.debug('New message: %s' % vars(msg))
        user = UserTelegram(protocol, msg.user)
        self.msg = msg
        if self.is_groupchat:
            groupchat = GroupChatTelegram(protocol, msg.groupname, int(self.msg.groupid))
        else:
            groupchat = None
        if protocol.bot is None and self.msg.ownmsg:
            protocol.bot = user
        # HACK: Mark messages from historic
        if self.protocol.nekbot.start_datetime > (datetime.now() + timedelta(seconds=10)) and \
                        self.msg.timestamp != datetime.now().strftime("%H:%M"):
            self.historical = True
        super(MessageTelegram, self).__init__(protocol, msg.message, user, groupchat)

    def reply(self, body, notice=False):
        self.protocol.tg.msg(self.msg.reply.cmd, self.protocol.prepare_message(body))

    @property
    def is_from_me(self):
        return self.msg.ownmsg

    @property
    def is_groupchat(self):
        return self.msg.groupname is not None


__author__ = 'nekmo'

from nekbot.protocols import User


class UserTelegram(User):
    def __init__(self, protocol, user):
        self.user = user
        User.__init__(self, protocol, user.name, user.id)

    def send_message(self, body, notice=False):
        self.protocol.tg.msg('user#%s' % self.id, self.protocol.prepare_message(body))